@@ -5,20 +5,21 @@ extract_r_source <- function(filename, lines, error = identity) {
     return(lines)
   }
 
+  # mask non-source lines by NA, but keep total line count identical so the line number for EOF is correct, see #1400
+  output <- rep.int(NA_character_, length(lines))
+
   chunks <- tryCatch(get_chunk_positions(pattern = pattern, lines = lines), error = error)
   if (inherits(chunks, "error") || inherits(chunks, "lint")) {
     assign("e", chunks,  envir = parent.frame())
     # error, so return empty code
-    return(character())
+    return(output)
   }
 
   # no chunks found, so just return the lines
   if (length(chunks[["starts"]]) == 0L || length(chunks[["ends"]]) == 0L) {
-    return(character())
+    return(output)
   }
 
-  # there is no need to worry about the lines after the last chunk end
-  output <- rep.int(NA_character_, max(chunks[["ends"]] - 1L))
   Map(
     function(start, end) {
       output[seq(start + 1L, end - 1L)] <<- lines[seq(start + 1L, end - 1L)]
@@ -30,6 +31,10 @@ extract_r_source <- function(filename, lines, error = identity) {
 }
 
 get_knitr_pattern <- function(filename, lines) {
+  # Early return if the source code is parseable as plain R code.
+  # Otherwise, R code containing a line which matches any knitr pattern will be treated as a knitr file.
+  # See #1406 for details.
+  if (parsable(lines)) return(NULL)
   pattern <- ("knitr" %:::% "detect_pattern")(lines, tolower(("knitr" %:::% "file_ext")(filename)))
   if (!is.null(pattern)) {
     knitr::all_patterns[[pattern]]
